{-# OPTIONS_GHC -fallow-overlapping-instances #-}
module Hawl.HaskellDB.Util where

import Database.HaskellDB
import Database.HaskellDB.Database
import Database.HaskellDB.PrimQuery (TableName)
import Database.HaskellDB.Query
import Database.HaskellDB.DBLayout

import Control.Monad (liftM)

query1 :: GetRec t r => Database -> Query (Rel t) -> IO (Record r)
query1 db q = query db q >>= result1

query01 :: GetRec t r => Database -> Query (Rel t) -> IO (Maybe (Record r))
query01 db q = query db q >>= result01

intQuery :: ProjectExpr e => Database -> Query (e Int) -> IO [Int]
intQuery db = liftM (map (!intField)) . query db . (>>= project . (intField <<))

intQuery1 :: ProjectExpr e => Database -> Query (e Int) -> IO Int
intQuery1 db iq = intQuery db iq >>= result1

result1 :: Monad m => [a] -> m a
result1 rs = case rs of
               []  -> fail "Query returned no results."
               [r] -> return r
               _   -> fail "Query returned more than one result."

result01 :: Monad m => [a] -> m (Maybe a)
result01 rs = case rs of
                []  -> return Nothing
                [r] -> return $ Just r
                _   -> fail "Query returned more than one result."

countField db q field = intQuery1 db $ liftM (count . (!field)) $ q

updateField db tab field start upd = 
    do
    mr <- get
    case mr of 
              Nothing -> do
                         insert db tab start
                         mr <- get
                         case mr of
                                 Nothing -> fail $ "updateField: Insert didn't work"
                                 Just x -> return x
              Just x -> do
                        let x' = upd x
                        update db tab (\r -> constant True) (\r -> field <<- x')
                        return x'
  where
  get = do
        rs <- query db $ do
                         t <- table tab
                         project (field << t!field)
        case rs of
                []  -> return Nothing
                [n] -> return $ Just (n!field)
                _   -> fail $ "updateField: Multiple records in " ++ tableName tab

tableName :: Table t -> TableName
tableName (Table n _) = n

--
-- * Generic record fields.
--

data IntField = IntField

instance FieldTag IntField where fieldName _ = "intfield"

intField :: Attr IntField Int
intField = mkAttr IntField
