module Hawl.Util where

import Data.Char
import Data.Monoid

--
-- * String utilities
--

isNumber :: String -> Bool
isNumber = all isDigit

--
-- * Function utilities
--

both :: (a -> b) -> (b -> b -> c) -> a -> a -> c
both f g x y = g (f x) (f y)

compose2 :: (c -> d) -> (a -> b -> c) -> a -> b -> d
compose2 f g x y = f (g x y)

--
-- * Equality and ordering functions
--

isEQ :: Ordering -> Bool
isEQ EQ = True
isEQ _  = False

ordEq :: (a -> a -> Ordering) -> a -> a -> Bool
ordEq = compose2 isEQ

eqBy :: Eq b => (a -> b) -> a -> a -> Bool
eqBy f = both f (==)

compareBy :: Ord b => (a -> b) -> a -> a -> Ordering
compareBy f = both f compare

--
-- * Monad utilities
--

whenMonoid :: (Monad m, Monoid a) => m Bool -> m a -> m a
whenMonoid p x = do b <- p
                    if b then x else return mempty

orM :: Monad m => m Bool -> m Bool -> m Bool
orM x y = do b <- x
             if b then return True else y

maybeM :: Monad m => String -> Maybe a -> m a
maybeM s = maybe (fail s) return

readM :: (Read a, Monad m) => String -> m a
readM s = readSM reads s

readSM :: Monad m => ReadS a -> String -> m a
readSM f s = case f s of
                      [] -> fail $ "No parse of " ++ show s
                      [(x,[])] -> return x
                      [(_,_)]  -> fail $ "Junk at end of " ++ show s
                      _  -> fail $ "Ambiguous parse of " ++ show s
