module Hawl.Util.Time where

import Control.Monad.Trans (MonadIO, liftIO)
import System.Time
import System.Time.Parse
import System.Locale (defaultTimeLocale)

--
-- * Time utilities
--

type Minutes = Int

timeNow :: MonadIO m => m CalendarTime
timeNow = liftIO (getClockTime >>= toCalendarTime)

addToCalendarTime :: TimeDiff -> CalendarTime -> CalendarTime
addToCalendarTime d t = toUTCTime $ addToClockTime d $ toClockTime t

minutesBefore :: Minutes -> CalendarTime -> CalendarTime
minutesBefore m t = addToCalendarTime (noTimeDiff { tdMin = m} ) t

epoch :: CalendarTime
epoch = CalendarTime {
		      ctYear = 1970,
		      ctMonth = January,
		      ctDay = 1,
		      ctHour = 0,
		      ctMin = 0,
		      ctSec = 0,
		      ctPicosec = 0,
		      ctWDay = Thursday,
		      ctYDay = 1,
		      ctTZName = "UTC",
		      ctTZ = 0,
		      ctIsDST = False
		     }

theEndOfTime :: CalendarTime
theEndOfTime = CalendarTime {
		             ctYear = 2^31-1,
		             ctMonth = December,
		             ctDay = 31,
		             ctHour = 23,
		             ctMin = 59,
		             ctSec = 59,
		             ctPicosec = 0,
		             ctWDay = Tuesday,
		             ctYDay = 365,
		             ctTZName = "UTC",
		             ctTZ = 0,
		             ctIsDST = False
                            }

monthToInt :: Month -> Int
monthToInt m = fromEnum m + 1

intToMonth :: Int -> Month
intToMonth x = toEnum (x-1)

showMonthInt :: Int -> String
showMonthInt = show . intToMonth

iso8601DateTimeFormat :: String
iso8601DateTimeFormat = "%Y-%m-%d %H:%M:%S"

parseISO8601DateTime :: Monad m => String -> m CalendarTime
parseISO8601DateTime x = 
    case parseCalendarTime defaultTimeLocale iso8601DateTimeFormat x of
        Nothing -> fail $ "Error parsing date: " ++ show x
        Just d  -> return d

showISO8601DateTime :: CalendarTime -> String
showISO8601DateTime = 
    formatCalendarTime defaultTimeLocale iso8601DateTimeFormat
